function [piest, info] = estimatepi(algorithm, params)
%ESTIMATEPI Estimates the value of the constant PI with several series-based algorithms
%   PIEST = ESTIMATEPI(ALGORITHM, PARAMS)
% Inputs:
%   ALGORITHM   - the algorithm (series) name, one of: 'madhava', 'newton', 
%               'ramanujan', 'chudnovsky', see [1]
%   PARAMS      - parameters structure, with fields:
%       K           - term to which the series should be computed
% Outputs:
%   PIEST       - the estimated value of PI
%   INFO        - structure containing additional information about the run, with fields:
%       exectime    - execution time
%
% References:
% [1] http://en.wikipedia.org/wiki/Numerical_approximations_of_?

% Copyright:    2010 Delft Center for Systems & Control (TUDelft), and Lucian Busoniu
% Author:       Lucian Busoniu
% Version:      1.0
% 
% Please do not redistribute this code without permission from the authors. If you wish to
% redistribute, contact one of: Robert Babuska, Lucian Busoniu, Ivo Grondman, Gabriel Lopes

if nargin < 1, algorithm = 'ramanujan'; end;
if nargin < 2, params = struct; end;    % so that defaults are installed for every field

% default parameter values
PARAMS.K = 5;   % expand series up to this term

params = parsestruct(params, PARAMS);

% compute pi according to algorithm
tic;
% each algorithm uses a series with K+1 terms, so we might as well initialize the series here
piseries = nan(params.K+1, 1);
switch algorithm,
    case 'madhava',
        for k = 0:params.K,
            piseries(k+1) = (-1/3)^k/(2*k+1);
        end;
        piest = sqrt(12) * sum(piseries);
        
    case 'newton',
        for k = 0:params.K,
            piseries(k+1) = 2^k * factorial(k)^2 / factorial(2*k+1);
        end;
        piest = 2 * sum(piseries);
        
    case 'ramanujan',
        for k = 0:params.K,
            piseries(k+1) = factorial(4*k) * (1103 + 26390 * k) / (factorial(k)^4 * 396^(4*k));
        end;
        piest = ((2 * sqrt(2) / 9801) * sum(piseries))^(-1);
        
    case 'chudnovsky',
        for k = 0:params.K,
            piseries(k+1) = (-1)^k * factorial(6*k) * (13591409 + 545140134 * k) ...
                / factorial(3*k) / factorial(k)^3 / 640320^(3*k+3/2);
        end;
        piest = (12 * sum(piseries))^(-1);
        
    otherwise,
       error('Algorithm [%s] unknown.', algorithm);
end;

% determine elapsed time = execution time
info.exectime = toc;

